# Gather all the arguments recognized by the command line


class Commands:
    """
    Secrets Guard commands.
    """

    HELP = "help"
    VERSION = "version"
    GIT_INIT = "init"
    GIT_PUSH = "push"
    GIT_PULL = "pull"
    CREATE_STORE = "create"
    DESTROY_STORE = "destroy"
    LIST_STORES = "list"
    SHOW_STORE = "show"
    CHANGE_STORE_KEY = "key"
    CLEAR_STORE = "clear"
    ADD_SECRET = "add"
    GREP_SECRET = "grep"
    REMOVE_SECRET = "remove"
    MODIFY_SECRET = "modify"


class Options:
    """
    Secrets Guard options (prefixed with --).
    """

    STORE_FIELDS = "fields"
    STORE_KEY = "key"
    STORE_PATH = "path"
    SECRET_DATA = "data"

    BRANCH = "branch"
    MESSAGE = "message"

    VERBOSE = "verbose"
    NO_COLOR = "no-color"
    NO_TABLE = "no-table"
    NO_KEYRING = "no-keyring"


class SecretAttributes:
    """
    Recognized secrets' attributes.
    """
    HIDDEN = "h"
    MANDATORY = "m"



from secrets_guard.utils import list_to_str, dict_of_lists_to_str


class Args:
    """
    Encapsulated parsed arguments.
    This contains
    1) The command (e.g. create/show/...)
    2) Positional arguments (e.g. the id for remove <ID>)
    3) Keyword arguments (e.g. --key <KEY>)
    """

    def __init__(self):
        self.command = None
        self.args = []
        self.kwargs = {}

    def __str__(self):
        s = "COMMAND: " + (self.command if self.command is not None else "<none>") + "\n" + \
            "Args: " + list_to_str(self.args) + "\n" + \
            "Kwargs: " + dict_of_lists_to_str(self.kwargs)

        return s

    def add_kwarg_param(self, arg, param):
        """
        Adds a param to a keyword argument.
        :param arg: the argument
        :param param: the param to add
        """
        if arg not in self.kwargs:
            self.kwargs[arg] = []

        self.kwargs[arg].append(param)

    def kwarg_params(self, arg, default=None):
        """
        Returns the params of the given keyword argument.
        :param arg: the argument
        :param default: the value to return if the argument does not exist
        :return: the params of the argument or the default value
        """
        return self.kwargs.get("--" + arg, default)

    def kwarg_param(self, arg, default=None):
        """
        Returns the first param of the given keyword argument.
        :param arg: the argument
        :param default: the value to return if the argument does not exist
        :return: the params of the argument or the default value
        """
        real_arg = "--" + arg
        if real_arg not in self.kwargs:
            return default

        params = self.kwargs.get("--" + arg)
        if not params:
            return None
        if len(params) <= 0:
            return None
        return params[0]

    def has_kwarg(self, arg):
        """
        Returns whether the given keyword argument is present.
        :param arg: the argument
        :return: whether the argument is present
        """

        return ("--" + arg) in self.kwargs



import logging

from git import Repo


def git_progress_handler(_op_code_, cur_count, max_count=None, message=''):
    print("Progress: {} / {} {}".format(cur_count, max_count, message))


def print_if_valid(something):
    if something:
        print(something)


# def init(local_path):
#     """
#     Initializes the local path as a git repository (git init)
#     :param local_path: the local path of the repository
#     :return: whether the path as been initialized as a git repository
#     """
#     if not local_path:
#         logging.error("Local path must be specified")
#         return False
#
#     logging.debug("Locating repo at path: %s", local_path)
#     repository = Repo(local_path)
#
#     if not repository:
#         logging.error("Not a git repository")
#         return False
#
#     repository.git.init()
#
#     return True


def push(local_path, commit_message, remote_branch=None):
    """
    Commits and push the local path (containing the stores) to the remote git branch
    :param local_path local repository path
    :param remote_branch remote branch name
    :param commit_message the commit message
    :return: whether the push has been performed
    """
    if not local_path:
        logging.error("Local path must be specified")
        return False

    if not commit_message:
        logging.error("A commit message must be specified")
        return False

    # if not remote_branch:
    #     logging.error("Remote branch must be specified")
    #     return False

    logging.debug("Locating repo at path: %s", local_path)
    repository = Repo(local_path)

    if not repository:
        logging.error("Not a git repository, cannot push")
        return False

    logging.debug("Adding . to stage")
    print_if_valid(repository.git.add("."))

    # Before commit, check if commit is needed, otherwise
    # an exception will be thrown

    repo_status = repository.git.status("-s")
    logging.debug("git status -s report is:\n%s", repo_status)
    if repo_status:
        logging.debug("Committing with message: %s", commit_message)
        print_if_valid(repository.git.commit("-m", commit_message))
        # repository.index.commit(commit_message)
    else:
        logging.debug("No commit is needed")

    logging.debug("Will push to branch %s", remote_branch)

    # remote = repository.remote(name=remote_branch)
    # if not remote or not remote.exists():
    #     logging.error("Cannot find remote branch named %s", remote_branch)
    #     return False
    # pushed = remote.push(progress=git_progress_handler)

    logging.debug("Really invoking push()")

    if remote_branch:
        print_if_valid(repository.git.push(remote_branch))
    else:
        print_if_valid(repository.git.push())

    return True


def pull(local_path, remote_branch):
    """
    Pulls from the remote git branch to the local path of the stores
    :param local_path local repository path
    :param remote_branch remote branch name
    :return: whether the pull has been performed
    """
    if not local_path:
        logging.error("Local path must be specified")
        return False

    # if not remote_branch:
    #     logging.error("Remote branch must be specified")
    #     return False

    logging.debug("Locating repo at path: %s", local_path)
    repository = Repo(local_path)

    if not repository:
        logging.error("Not a git repository, cannot pull")
        return False

    logging.debug("Will pull from branch %s", remote_branch)

    # remote = repository.remote(name=remote_branch)
    # if not remote or not remote.exists():
    #     logging.error("Cannot find remote branch named %s", remote_branch)
    #     return False

    logging.debug("Really invoking pull()")

    if remote_branch:
        print_if_valid(repository.git.pull(remote_branch))
    else:
        print_if_valid(repository.git.pull())

    return True



import logging
import random
import re
import string
import sys
from functools import reduce
from getpass import getpass
from sys import exit

from secrets_guard.consts import AnsiColor

PROMPT_DOUBLE_CHECK_FAILED = -1


def eprint(*args, **kwargs):
    """
    Prints to stderr.
    :param args: arguments argument to pass to print()
    :param kwargs: keyword argument to pass to print()
    """
    print(*args, file=sys.stderr, **kwargs)


def terminate(message, exit_code=0):
    """
    Exit gracefully with the given message and exit code.
    :param message: the message to print to stdout before exit
    :param exit_code: the exit code
    """
    print(message)
    exit(exit_code)


def abort(message, exit_code=-1):
    """
    Exit ungracefully with the given message and exit code.
    :param message: the message to print to stderr before exit
    :param exit_code: the exit code
    """
    logging.error(message)
    eprint(message)
    exit(exit_code)


def is_string(s):
    """
    Returns whether the object is a string.
    :param s: the object
    :return: whether is a string
    """
    return type(s) is str


def is_list(l):
    """
    Returns whether the object is a list.
    :param l: the object
    :return: whether is a list
    """
    return isinstance(l, list)


def is_empty_string(s):
    """
    Returns whether the given string is empty
    :param s: the string
    :return: whether the string is empty
    """
    return s == ""


def list_to_str(l, sep=", "):
    """
    Joins the elements of the list using the given separator.
    :param l: the list
    :param sep: the separator to use for join elements
    :return: the joined string
    """
    if l is None:
        return ""
    return sep.join(l)


def dict_of_lists_to_str(d, sep=": "):
    """
    Returns a string representation of a dictionary of lists.
    :param d: the dictionary
    :param sep: the separator
    :return: the string representation
    """
    if d is None:
        return "<none>"
    s = ""
    for k in d:
        s += k
        if len(d[k]) > 0:
            s += sep + list_to_str(d[k]) + " | "
        s += "; "
    return s


def keyval_list_to_dict(l):
    """
    Converts a list of key=value strings to a dictionary.
    :param l: the list
    :return: the dictionary
    """
    d = {}
    for e in l:
        keyval = e.split("=", 2)
        if len(keyval) != 2:
            return None
        d[keyval[0]] = keyval[1]
    return d


def random_string(length=10, alphabet=string.ascii_lowercase):
    """
    Generate a random string of the given length.
    :param length: the length of the random string to generate
    :param alphabet: a string containing the alphabet to use for the generation.
                     Default is string.ascii_lowercase
    :return: the randomly generated string
    """
    return ''.join(random.choice(alphabet) for _ in range(length))


def enumerate_data(headers, data, enum_field_name="ID"):
    """
    Returns the enumerated headers and data.
    The field enum_field_name will be added to the headers, and the same field
    will be set for the items of data equal to their index.
    :param headers: the headers
    :param data: the data
    :param enum_field_name: the field name
    :return: the enumerated headers and the enumerated data
    """
    enum_headers = [enum_field_name] + headers
    enum_data = []
    for i, d in enumerate(data):
        enum_data.append(d)
        if enum_field_name not in d:
            enum_data[i][enum_field_name] = i
    return enum_headers, enum_data


def tabulate_enum(headers, data, enum_field_name="ID"):
    """
    Returns a string representation of the given data list using the given headers.
    Furthermore add a column used for enumerate rows.
    :param headers: a list of strings representing the headers
    :param data: a list of objects with the properties specified in headers
    :param enum_field_name: the name of the header used for enumerate the rows
    :return: the table string of the data
    """
    enum_headers, enum_data = enumerate_data(headers, data, enum_field_name)

    return tabulate(enum_headers, enum_data)


def tabulate(headers, data):
    """
    Returns a string representation of the given data list using the given headers.
    :param headers: a list of strings representing the headers
    :param data: a list of objects with the properties specified in headers
    :return: the table string of the data
    """

    ansi_escaper = re.compile("\x1B\\[[0-9]+m")

    def escaped_text_lenth(text):
        # Remove '\33[??m' codes from text
        ansi_colors = ansi_escaper.findall(text)
        ansi_colors_len = 0
        if ansi_colors and len(ansi_colors) > 0:
            ansi_colors_len = reduce(lambda c1, c2: len(c1) + len(c2), ansi_colors)
        return len(text) - ansi_colors_len

    HALF_PADDING = 1
    PADDING = 2 * HALF_PADDING

    out = ""

    max_lengths = {}

    # Compute max length for each field
    for h in headers:
        m = len(h)
        for d in data:
            if h in d:
                m = max(m, escaped_text_lenth(str(d[h])))
        max_lengths[h] = m

    def separator_row(newline=True):
        s = ""
        for hh in headers:
            s += "+" + ("-" * (max_lengths[hh] + PADDING))
        s += "+"

        if newline:
            s += "\n"

        return s

    def data_cell(filler):
        return (" " * HALF_PADDING) + filler() + (" " * HALF_PADDING)

    def data_cell_filler(text, fixed_length):
        # Consider ANSI color before pad
        return text.ljust(fixed_length + (len(text) - escaped_text_lenth(text)))

    # Row
    out += separator_row()

    # Headers
    for h in headers:
        out += "|" + data_cell(lambda: data_cell_filler(h, max_lengths[h]))
    out += "|\n"

    # Row
    out += separator_row()

    # Data
    for d in data:
        for dh in headers:
            out += "|" + data_cell(
                lambda: data_cell_filler((str(d[dh]) if dh in d else " "), max_lengths[dh]))
        out += "|\n"

    # Row
    out += separator_row(newline=False)

    return out


def prompt(prompt_text,
           secure=False,
           double_check=False,
           double_check_prompt_text=None,
           double_check_failed_message=None,
           until_valid=False):
    """
    Asks for input, eventually hiding what's being written, until the input is valid.
    :param prompt_text: the text to show
    :param secure: whether the input should be secret
    :param double_check: whether the input should be double checked (only if secure is true)
    :param double_check_prompt_text: the text to show for the double check.
                                     If not specified, the prompt_text is used instead
    :param double_check_failed_message: the text to show if the double check fails
    :param until_valid: prompt again until a valid string is inserted (eventually double checking)
    :return: the (valid) text inserted by the user
    """

    while True:
        s = _prompt(prompt_text, secure, double_check, double_check_prompt_text)

        # If s is valid, we are ok returning it anyway
        if s and s != PROMPT_DOUBLE_CHECK_FAILED:
            return s

        # At this point s is either empty or the double check failed

        if s == PROMPT_DOUBLE_CHECK_FAILED:
            # Double check failed: never allow PROMPT_DOUBLE_CHECK_FAILED.
            if double_check_failed_message:
                print(double_check_failed_message)
            # will re-prompt
        else:
            # The string is invalid: decide whether re-prompt or
            # allow empty string depending on until_valid.
            if not until_valid:
                return s
            # else: will re-prompt


def _prompt(prompt_text, secure=False, double_check=False, double_check_prompt_text=None):
    """
    Asks for input, eventually hiding what's being written, and eventually doing a double check.
    :param prompt_text: the text to show
    :param secure: whether the input should be secret
    :param double_check: whether the input should be double checked (only if secure is true)
    :param double_check_prompt_text: the text to show for the double check.
                                     If not specified, the prompt_text is used instead
    :return: the text inserted by the user, or None if the double_check failed
    """
    if not secure:
        # Plain
        return input(prompt_text)

    # Secure
    s1 = getpass(prompt_text)
    if not double_check:
        return s1

    # Double check
    if not double_check_prompt_text:
        double_check_prompt_text = prompt_text

    s2 = getpass(double_check_prompt_text)

    return s1 if s1 == s2 else PROMPT_DOUBLE_CHECK_FAILED


def insert_into_string(source, insert, pos):
    """
    Inserts a string into another string and returns the new string
    :param source: the source string
    :param insert: the string to insert
    :param pos: the position the string will be inserted to
    :return: the composed string
    """
    return source[:pos] + insert + source[pos:]


def highlight(text, startpos, endpos, color=AnsiColor.RED):
    """
    Highlights the text by insert the given ANSi color (default: red).
    :param text: the text to highlight
    :param startpos: the starting highlight position
    :param endpos: the ending highlight position
    :param color: a valid ANSI color escape sequence
    :return: the highlighted text
    """

    highlighted_text = text

    logging.debug("Highlighting %s from %d to %d", text, startpos, endpos)
    highlighted_text = insert_into_string(highlighted_text, AnsiColor.RESET, endpos)
    highlighted_text = insert_into_string(highlighted_text, color, startpos)

    return highlighted_text



import base64
import hashlib
import logging

from Crypto import Random
from Crypto.Cipher import AES
from Crypto.Util.Padding import pad, unpad


def aes_key(key):
    """
    Generate a valid AES key from a string.
    :param key: the key from which generate an AES key
    :return: the AES key
    """
    return hashlib.sha256(key.encode()).digest()


def aes_encrypt(plaintext, iv, key, is_plain_key=True):
    """
    Encrypts a text using the given IV and key
    (saving the IV at the beginning of the file).
    :param plaintext: the text to encrypt
    :param iv: the AES initialization vector
    :param key: the key
    :param is_plain_key: whether the key is plaintext or is already hashed
    :return: the encrypted content
    """
    key = aes_key(key) if is_plain_key else key

    padded_text = pad(bytes(plaintext, encoding="utf-8"), AES.block_size)
    cipher = AES.new(key, mode=AES.MODE_CBC, IV=iv)
    return base64.b64encode(iv + cipher.encrypt(padded_text))


def aes_decrypt(encrypted_content, key, is_plain_key=True):
    """
    Decrypts an encrypted content using the given key
    (looking for the IV at the beginning of the file).
    :param encrypted_content: the encrypted content (as b64 text)
    :param key: the key
    :param is_plain_key: whether the key is plaintext or is already hashed
    :return: the plaintext
    """

    key = aes_key(key) if is_plain_key else key

    decoded_content = base64.b64decode(encrypted_content)
    iv = decoded_content[:AES.block_size]
    body = decoded_content[AES.block_size:]

    cipher = AES.new(key, mode=AES.MODE_CBC, IV=iv)
    return unpad(cipher.decrypt(body), AES.block_size).decode("utf-8")


def aes_encrypt_file(path, key, content, is_plain_key=True):
    """
    Encrypts a file using AES (saving the IV at the beginning of the file).
    :param path: the path where store the encrypted file
    :param key: the key
    :param content: the plaintext
    :param is_plain_key: whether the key is plaintext or is already hashed
    :return: whether the file has been encrypted successfully
    """

    try:
        with open(path, "wb") as file:
            iv = Random.new().read(AES.block_size)
            encrypted_content = aes_encrypt(content, iv, key, is_plain_key=is_plain_key)
            file.write(encrypted_content)
        return True
    except OSError as e:
        logging.error("AES I/O error", e)
        return False


def aes_decrypt_file(path, key, is_plain_key=True):
    """
    Decrypts a file using AES (looking for the IV at the beginning of the file).
    :param path: the path of the file to decrypt
    :param key: the key
    :return: whether the file has been decrypted successfully
    :param is_plain_key: whether the key is plaintext or is already hashed
    """
    try:
        with open(path, "rb") as file:
            encrypted_content = file.read()
            plaintext = aes_decrypt(encrypted_content, key, is_plain_key=is_plain_key)
            return plaintext
    except OSError as e:
        logging.error("AES I/O error", e)
        return False
    except UnicodeError:
        logging.error("AES decrypt error (invalid key?)")
        return False



import datetime
import logging
import os
import sys
import traceback

from secrets_guard import gitsync
from secrets_guard.args import Args
from secrets_guard.cli import Options, Commands, SecretAttributes
from secrets_guard.conf import Conf, LoggingLevels
from secrets_guard.keyring import keyring_get_key, keyring_put_key, keyring_has_key, keyring_del_key
from secrets_guard.store import Store, StoreField
from secrets_guard.utils import keyval_list_to_dict, abort, terminate, is_empty_string, prompt, is_string

HELP = """NAME 
    secrets - encrypt and decrypt private information (such as passwords)

SYNOPSIS
    secrets <COMMAND> [COMMAND_OPTIONS] [GENERAL_OPTIONS]
    
DESCRIPTION
    Stores and retrieves encrypted data to/from files.
    
    Each command can be used either in batch or interactive mode;
    each mandatory but not specified argument will be required interactively.
    
    One of the following command must be specified:
    
UTILITY COMMANDS

    help
        Shows this help message.
        
    version
        Shows the version number.
        
GLOBAL COMMANDS
                        
    list [--path <PATH>]
        List the names of the stores found at the path specified
        by --path (or at the default one if not specified).
    
        e.g. secrets list
 
STORE COMMANDS
        
    create [<STORE_NAME>] [--fields FIELDS] [--path <PATH>] [--key <STORE_KEY>]
        Creates a new store at the given path using the given key.
        The FIELDS must be expressed as a space separated list of field names.
        
        Furthermore some attributes can be expressed for the fields by appending
        "+<attr_code_1><attr_code_2>..." after the field name.
        
        The available attributes are
        1) h: hidden (the user input is not shown)
        2) m: mandatory (the field must contain a non empty string)
        
        e.g. secrets create password --fields Site Account Password Other --key mykey
        e.g. secrets create password --fields Site+m Account+m Password+mh Other --mykey
        
    destroy [<STORE_NAME>] [--path <PATH>]
        Destroys the store at the given path.
        
        e.g. secrets destroy password

    key [<STORE_NAME>] [<NEW_STORE_KEY>] [--path <PATH>] [--key <STORE_KEY>]
        Changes the key of the store from STORE_KEY to NEW_STORE_KEY.
        
        e.g. secrets key newkey --key currentkey
        
    clear [<STORE_NAME>] [--path <PATH>] [--key <STORE_KEY>]
        Clears the content (all the secrets) of a store.
        The model is left unchanged.
        
    show [<STORE_NAME>] [--path <PATH>] [--key <STORE_KEY>] [--no-table]
        Decrypts and shows the content of an entire store.
        
        e.g. secrets show password --key mykey
            
    grep [<STORE_NAME>] [<SEARCH_PATTERN>] [--path <PATH>] [--key <STORE_KEY>] [--no-color] [--no-table]
        Performs a regular expression search between the data of the store.
        The SEARCH_PATTERN can be any valid regular expression.
        The matches will be highlighted unless --no-color is specified.
        
        e.g. secrets grep password MyPass --key mykey
        e.g. secrets grep password "^My.*word" --key mykey
        
SECRET COMMANDS
        
    add [<STORE_NAME>] [--data DATA] [--path <PATH>] [--key <STORE_KEY>]
        Inserts a new secret into a store.
        The DATA must be expressed as a key=value list where the key should
        be a field of the store.
        
        e.g. secrets add password --data Site="Megavideo" Account="me@gmail.com" Password="MyPassword" --key mykey

    remove [<STORE_NAME>] [<SECRET_IDS>*] [--path <PATH>] [--key <STORE_KEY>]
        Removes the secret(s) with the given SECRET_IDS from the store.
        The SECRET_IDS should be retrieved using the secrets grep command.
        
        e.g. secrets remove password 12
        e.g. secrets remove password 12 14 15 7 11
    
    modify [<STORE_NAME>] [<SECRET_ID>] [--data DATA] [--path <PATH>] [--key <STORE_KEY>]
        Modifies the secret with the given SECRET_ID using the given DATA.
        The DATA must be expressed as a key=value list.
    
        e.g. secrets modify password 11 --data Password="MyNewPassword" --key mykey
               
GIT COMMANDS

    push [--path <PATH>] [--message <COMMIT_MESSAGE>] [--remote <REMOTE_NAME>]
        Commits and pushes to the remote git repository.
        Actually performs "git add ." , "git commit -m 'COMMIT_MESSAGE'" and
        "git push [REMOTE_NAME]" on the given path.
        Note that the action is related to the whole repository, 
        not a particular store.

        If the COMMIT_MESSAGE is not specified, a default commit message 
        will be created.
        If the REMOTE_NAME is not specified, the default one 
        (if set, e.g. via --set-upstream) will be used.
        The credentials might be required by the the invoked git push routine.
        
        e.g. secrets push
        e.g. secrets push --remote origin
        e.g. secrets push --remote bitbucket --message "Added Google password"
          
    pull [--remote <REMOTE_NAME>] [--path <PATH>]
        Pull from the remote git branch.
        Note that the action is related to the whole repository, 
        not a particular store.

        e.g. secrets pull --remote origin

GENERAL OPTIONS
    --verbose
        Prints debug statements.
    
    --no-keyring
        Do not use the keyring for retrieve the password.
        By default a password used for open a store is cached in the keyring
        for further uses."""


def init_logging(parsed_args):
    """ Initializes the logging. """

    logging.addLevelName(LoggingLevels.TRACE, "TRACE")

    def trace(message, *args, **kws):
        if logging.getLogger().isEnabledFor(LoggingLevels.TRACE):
            logging.log(LoggingLevels.TRACE, message, *args, **kws)

    logging.trace = trace
    logging.Logger.trace = trace

    logging.basicConfig(level=Conf.LOGGING_LEVEL,
                        format="[%(levelname)s] %(asctime)s %(message)s",
                        datefmt='%d/%m/%y %H:%M:%S',
                        stream=sys.stdout)

    if not parsed_args.has_kwarg(Options.VERBOSE):
        logging.disable()

# =====================
# ===== ARGUMENTS =====
# =====================


def parse_arguments(arguments):
    """
    Parses the argument list.
    :param arguments: the arguments to parse (sys.argv)
    :return: the parsed arguments
    """
    # logging.debug("Parsing arguments %s", arguments)

    parsed_args = Args()

    if len(arguments) < 1:
        abort("Error: the command must be specified")

    # Parse command

    command_request = arguments[0]

    for command in Commands.__dict__.values():
        if command == command_request:
            parsed_args.command = command

    if parsed_args.command is None:
        abort("Error: unknown command '%s'" % command_request)

    # Parse position/keyword arguments

    i = 1
    current_args_stream = parsed_args.args

    while i < len(arguments):
        arg = arguments[i]
        if not arg.startswith("--"):
            # Parameter of the current argument
            # logging.debug("Adding parameter %s to current argument", arg)
            current_args_stream.append(arg)
        else:
            # New argument
            # logging.debug("Found new argument: %s", arg)
            current_args_stream = []
            parsed_args.kwargs[arg] = current_args_stream
        i += 1

    return parsed_args


def obtain_positional_argument(parsed_args, index, prompt_text, secure=False, double_check=False, count=1):
    """
    Gets the positional argument at the given index from parsed_args
    or asks the user to input it if not present between parsed_args.
    :param parsed_args: the parsed arguments
    :param index: the index of the argument
    :param prompt_text: the text eventually prompted to the user
    :param secure: whether the input should be hidden
    :param double_check: whether double check the secure input
    :param count: how many arguments take
    :return: the obtained value
    """
    if len(parsed_args.args) > index:
        if not count:
            return parsed_args.args[index:]

        if count == 1:
            return parsed_args.args[index]

        return parsed_args.args[index:index + count]

    return prompt(prompt_text, secure=secure, double_check=double_check, until_valid=True)


def obtain_argument_params(parsed_args, argument, prompt_text, secure=False, double_check=False):
    """
    Gets the argument's params from parsed_args or asks the user to input it
    if not present between parsed_args.
    :param parsed_args: the parsed arguments
    :param argument: the name of the argument for which get the params
    :param prompt_text: the text eventually prompted to the user
    :param secure: whether the input should be hidden
    :param double_check: whether double check the secure input
    :return: the obtained value
    """
    params = parsed_args.kwarg_params(argument)

    if params:
        return params

    return prompt(prompt_text, secure=secure, double_check=double_check, until_valid=True)


def obtain_argument_param(parsed_args, argument, prompt_text, secure=False, double_check=False):
    """
    Gets the first argument's param from parsed_args or asks the user to input it
    if not present between parsed_args.
    :param parsed_args: the parsed arguments
    :param argument: the name of the argument for which get the params
    :param prompt_text: the text eventually prompted to the user
    :param secure: whether the input should be hidden
    :param double_check: whether double check the secure input
    :return: the obtained value
    """
    param = parsed_args.kwarg_param(argument)

    if param:
        return param

    return prompt(prompt_text, secure=secure, double_check=double_check, until_valid=True)


def obtain_optional_argument_param(parsed_args, argument, default_value=None):
    """
    Gets the first argument's param from parsed_args or returns the default value
    :param parsed_args: the parsed arguments
    :param argument: the name of the argument for which get the param
    :param default_value the value to return if the parameter is not present
    :return: the obtained value or the default value if the parameter if the argument
            is not present
    """
    param = parsed_args.kwarg_param(argument)

    if param:
        return param

    return default_value


def obtain_store_name(parsed_args):
    """
    Gets the store name if present in parsed_args or asks the user to input it
    if not present between parsed_args.
    :param parsed_args: the parsed arguments
    :return: the store name
    """
    return obtain_positional_argument(parsed_args, 0, "Store name: ") + Conf.STORE_EXTENSION


def obtain_store_key(parsed_args, keyring_store_nome=None):
    """
    Gets the store key if present in parsed_args or asks the user to input it
    if not present between parsed_args.
    :param parsed_args: the parsed arguments
    :param keyring_store_nome: the store name to use for eventually retrieve the
                                key from the keyring
    :return: the store key
    """
    # Check between arguments
    key = parsed_args.kwarg_param(Options.STORE_KEY)

    if key:
        return key

    # Check for cached key
    if keyring_store_nome:
        aeskey = keyring_get_key(keyring_store_nome)
        if aeskey:
            return aeskey

    return obtain_argument_param(parsed_args, Options.STORE_KEY,
                                 "Store key: ", secure=True, double_check=False)


def obtain_store_path(parsed_args, ensure_existence=True):
    """
    Gets the store path if present in parsed_args or asks the user to input it
    if not present between parsed_args.
    :param parsed_args: the parsed arguments
    :param ensure_existence: whether abort if the path does not exist
    :return: the store path
    """
    path = parsed_args.kwarg_param(Options.STORE_PATH, Conf.DEFAULT_SECRETS_PATH)
    if ensure_existence and not os.path.exists(path):
        abort("Error: path does not exist (%s)" % path)
    return path


def obtain_common_store_arguments(parsed_args, ensure_valid_path=True, allow_keyring=True):
    """
    Gets the store path, name and key if present in parsed_args or
    asks the user to input them if not present between parsed_args.
    :param parsed_args: the parsed arguments
    :param ensure_valid_path: whether abort if the path does not exist
    :param allow_keyring: whether the keyring should be used, unledd --no-keyring is specified
    :return: a tuple with path, name and key
    """

    use_keyring = allow_keyring and not parsed_args.has_kwarg(Options.NO_KEYRING)

    path = obtain_store_path(parsed_args, ensure_existence=ensure_valid_path)
    name = obtain_store_name(parsed_args)
    key = obtain_store_key(parsed_args, keyring_store_nome=name if use_keyring else None)

    return path, name, key, use_keyring


# =========================
# ======== COMMANDS =======
# =========================


def attempt_execute_command(command, error_message="Unexpected error occurred"):
    command_ok = False
    try:
        command_ok = command()
    except Exception as e:
        logging.warning("Caught exception: %s", e)
        logging.warning(traceback.format_exc())

    if not command_ok:
        abort(error_message)


def execute_help(_):
    terminate(HELP)


def execute_version(_):
    terminate(Conf.APP_NAME + " " + Conf.APP_VERSION)


def execute_create_store(parsed_args):
    store_path, store_name, store_key, use_keyring = \
        obtain_common_store_arguments(parsed_args,
                                      ensure_valid_path=False,
                                      allow_keyring=False)

    # Store fields
    raw_store_fields = parsed_args.kwarg_params(Options.STORE_FIELDS)

    if raw_store_fields is None:
        raw_store_fields = []
        f = None
        i = 1
        print("\nInsert store fields with format <name>[+<properties>].\n(Leave empty for end the fields insertion).")
        while not is_empty_string(f):
            f = input(str(i) + "° field: ")
            if not is_empty_string(f):
                raw_store_fields.append(f)
            i += 1

    store_fields = []
    for raw_field in raw_store_fields:
        field_parts = raw_field.split("+")
        fieldname = field_parts[0]
        fieldattributes = field_parts[1] if len(field_parts) > 1 else []

        store_fields.append(StoreField(
            fieldname,
            hidden=SecretAttributes.HIDDEN in fieldattributes,
            mandatory=SecretAttributes.MANDATORY in fieldattributes)
        )

    def do_create_store():
        store = Store(store_path, store_name, store_key)
        store.add_fields(*store_fields)
        return store.save()

    attempt_execute_command(
        do_create_store,
        error_message="Error: cannot create store"
    )


def execute_destroy_store(parsed_args):
    store_path = obtain_store_path(parsed_args)
    store_name = obtain_store_name(parsed_args)

    def do_destroy_store():
        store = Store(store_path, store_name)
        if not store.destroy():
            return False

        # Remind to delete the keyring
        keyring_del_key(store_name)

        return True

    attempt_execute_command(
        do_destroy_store,
        error_message="Error: cannot destroy store"
    )


def execute_list_stores(parsed_args):
    store_path = obtain_store_path(parsed_args, ensure_existence=False)

    if not os.path.exists(store_path):
        logging.warning("Store path does not exists")
        # Not an error, just no stores
        return

    for filename in os.listdir(store_path):
        # Consider only store files
        if filename.endswith(Conf.STORE_EXTENSION):
            # Remove the extension
            print(filename.rsplit(Conf.STORE_EXTENSION, 1)[0])


def execute_show_store(parsed_args):
    store_path, store_name, store_key, use_keyring = obtain_common_store_arguments(parsed_args)

    def do_show_store():
        store = Store(store_path, store_name, store_key)
        open_store(store, update_keyring=use_keyring)
        return store.show(table=not parsed_args.has_kwarg(Options.NO_TABLE))

    attempt_execute_command(
        do_show_store,
        error_message="Error: cannot show store"
    )


# def execute_git_init(parsed_args):
#     store_path = obtain_store_path(parsed_args)
#
#     def do_init():
#         logging.debug("Will in at %s", store_path)
#         return gitsync.init(store_path)
#
#     attempt_execute_command(
#         do_init,
#         error_message="Error: cannot init"
#     )


def execute_git_push(parsed_args):
    store_path = obtain_store_path(parsed_args)
    remote_branch = obtain_optional_argument_param(parsed_args, Options.BRANCH)
    commit_message = obtain_optional_argument_param(parsed_args, Options.MESSAGE)

    def do_push():
        nonlocal commit_message

        logging.debug("Will push %s to branch %s", store_path, remote_branch)

        if not commit_message:
            commit_message = "Committed on " + datetime.datetime.now().strftime("%H:%M:%S %d/%m/%Y")

        return gitsync.push(store_path, commit_message, remote_branch)

    attempt_execute_command(
        do_push,
        error_message="Error: cannot push"
    )


def execute_git_pull(parsed_args):
    store_path = obtain_store_path(parsed_args)
    remote_branch = obtain_optional_argument_param(parsed_args, Options.BRANCH)

    def do_pull():
        logging.debug("Will pull from branch %s to %s", store_path, remote_branch)
        return gitsync.pull(store_path, remote_branch)

    attempt_execute_command(
        do_pull,
        error_message="Error: cannot pull"
    )


def execute_change_store_key(parsed_args):
    store_path, store_name, store_key, use_keyring = obtain_common_store_arguments(parsed_args)
    new_store_key = obtain_positional_argument(parsed_args, 1, "New store key: ",
                                               secure=True, double_check=True)

    def do_change_store_key():
        store = Store(store_path, store_name, store_key)
        store.open()

        new_store = Store(store_path, store_name, new_store_key)
        new_store.clone_content(store)

        if not new_store.save():
            return False

        # Remind to delete the keyring
        keyring_del_key(store_name)

        return True

    attempt_execute_command(
        do_change_store_key,
        error_message="Error: cannot change store key"
    )


def execute_clear_store(parsed_args):
    store_path, store_name, store_key, use_keyring = obtain_common_store_arguments(parsed_args)

    def do_clear_store():
        store = Store(store_path, store_name, store_key)
        store.open()
        store.clear_secrets()
        return store.save()

    attempt_execute_command(
        do_clear_store,
        error_message="Error: cannot clear store"
    )


def execute_add_secret(parsed_args):
    store_path, store_name, store_key, use_keyring = obtain_common_store_arguments(parsed_args)

    # Secret data

    secret_data = parsed_args.kwarg_params(Options.SECRET_DATA)

    def do_add_secret():
        store = Store(store_path, store_name, store_key)
        open_store(store, update_keyring=use_keyring)

        if secret_data is None:
            secret = {}
            for f in store.fields:
                secret[f.name] = prompt(
                    f.name + ": ", secure=f.hidden,
                    double_check=True,
                    double_check_prompt_text=f.name + " again: ",
                    double_check_failed_message="Double check failed, please insert the field again",
                    until_valid=f.mandatory)
        else:
            secret = keyval_list_to_dict(secret_data)
            # If there are already some fields, ask only the mandatory
            # (since this is probably non interactive mode and we won't
            # block the execution)
            for f in store.fields:
                if f.mandatory:
                    has_mandatory = False
                    for secfield in secret:
                        if f.name.lower() == secfield.lower():
                            has_mandatory = True
                            break
                    if not has_mandatory:
                        secret[f.name] = prompt(
                            f.name + ": ", secure=f.hidden,
                            double_check=True,
                            double_check_prompt_text=f.name + " again: ",
                            double_check_failed_message="Double check failed, please insert the field again",
                            until_valid=f.mandatory)

        if not store.add_secrets(secret):
            return False

        return store.save()

    attempt_execute_command(
        do_add_secret,
        error_message="Error: cannot add secret"
    )


def execute_grep_secret(parsed_args):
    store_path, store_name, store_key, use_keyring = obtain_common_store_arguments(parsed_args)
    grep_pattern = obtain_positional_argument(parsed_args, 1, "Search pattern: ")

    def do_grep_secret():
        store = Store(store_path, store_name, store_key)
        open_store(store, update_keyring=use_keyring)
        return store.grep(grep_pattern,
                          colors=not parsed_args.has_kwarg(Options.NO_COLOR),
                          table=not parsed_args.has_kwarg(Options.NO_TABLE))

    attempt_execute_command(
        do_grep_secret,
        error_message="Error: cannot search for secrets"
    )


def execute_remove_secret(parsed_args):
    store_path, store_name, store_key, use_keyring = obtain_common_store_arguments(parsed_args)

    raw_secrets_ids = obtain_positional_argument(
        parsed_args, 1, "ID of the secret(s) to remove: ", count=None)

    # Convert to list if is a string (took with input())
    if is_string(raw_secrets_ids):
        raw_secrets_ids = raw_secrets_ids.split(" ")

    secret_ids = [int(sid) for sid in raw_secrets_ids]

    def do_remove_secret():
        store = Store(store_path, store_name, store_key)
        open_store(store, update_keyring=use_keyring)
        if not store.remove_secrets(*secret_ids):
            return False
        return store.save()

    attempt_execute_command(
        do_remove_secret,
        error_message="Error: cannot remove secrets with ID %s (index out of bound?)" % secret_ids
    )


def execute_modify_secret(parsed_args):
    store_path, store_name, store_key, use_keyring = obtain_common_store_arguments(parsed_args)
    secret_id = int(obtain_positional_argument(parsed_args, 1, "ID of the secret to modify: "))
    secret_data = parsed_args.kwarg_params(Options.SECRET_DATA)

    def do_modify_secret():
        store = Store(store_path, store_name, store_key)
        open_store(store, update_keyring=use_keyring)

        # Secret data
        if secret_data is None:
            secret_mod = {}

            secret = store.secret(secret_id)

            if not secret:
                abort("Error: invalid secret ID; index out of bound")

            logging.debug("Will modify secret %s", secret)

            print("Which field to modify?")
            choice = len(store.fields)

            max_length = 0

            for f in store.fields:
                max_length = max(len(f.name), max_length)

            while choice >= len(store.fields):
                for i, f in enumerate(store.fields):
                    s = str(i) + ") " + f.name.ljust(max_length)
                    if f.name in secret:
                        s += " (" + (secret[f.name] if not f.hidden else "*" * len(secret[f.name])) + ")"
                    print(s)
                choice = int(input(": "))

            changed_field = store.fields[choice]
            newval = prompt(
                "New value of '" + changed_field.name + "': ", secure=changed_field.hidden,
                double_check=True,
                double_check_prompt_text="New value of '" + changed_field.name + "' again: ",
                double_check_failed_message="Double check failed, please insert the field again",
                until_valid=changed_field.mandatory)

            secret_mod[changed_field.name] = newval
        else:
            secret_mod = keyval_list_to_dict(secret_data)

        if not store.modify_secret(secret_id, secret_mod):
            return False

        return store.save()

    attempt_execute_command(
        do_modify_secret,
        error_message="Error: cannot modify secret with ID %d (index out of bound?)" % secret_id
    )


def execute_command(parsed_args):
    if parsed_args is None or parsed_args.command is None:
        abort("Error: invalid arguments (command not specified)")

    dispatcher = {
        Commands.HELP: execute_help,
        Commands.VERSION: execute_version,
        # Commands.GIT_INIT: execute_git_init,
        Commands.GIT_PUSH: execute_git_push,
        Commands.GIT_PULL: execute_git_pull,
        Commands.CREATE_STORE: execute_create_store,
        Commands.DESTROY_STORE: execute_destroy_store,
        Commands.LIST_STORES: execute_list_stores,
        Commands.SHOW_STORE: execute_show_store,
        Commands.CHANGE_STORE_KEY: execute_change_store_key,
        Commands.CLEAR_STORE: execute_clear_store,
        Commands.ADD_SECRET: execute_add_secret,
        Commands.GREP_SECRET: execute_grep_secret,
        Commands.REMOVE_SECRET: execute_remove_secret,
        Commands.MODIFY_SECRET: execute_modify_secret
    }

    if parsed_args.command not in dispatcher:
        abort("Error: unknown command request '%s'" % parsed_args.command)
    logging.debug("Executing command '" + parsed_args.command + "'")

    try:
        dispatcher[parsed_args.command](parsed_args)
    except KeyboardInterrupt:
        logging.debug("Interrupted by user")
        exit(-1)


def open_store(store, update_keyring=True):
    if update_keyring:
        open_store_updating_keyring(store)
    else:
        store.open()


def open_store_updating_keyring(store):
    store.open()
    if not keyring_has_key(store.name):
        keyring_put_key(store.name, store.key, is_plain_key=store.has_plain_key())


def main():
    if len(sys.argv) <= 1:
        terminate(HELP)

    args = parse_arguments(sys.argv[1:])

    init_logging(args)

    logging.info("Executing script with arguments: \n%s", args)

    execute_command(args)


if __name__ == "__main__":
    main()



import os
from pathlib import Path


class LoggingLevels:
    CRITICAL = 50
    ERROR = 40
    WARNING = 30
    INFO = 20
    DEBUG = 10
    TRACE = 5


class Conf:
    APP_NAME = "secrets-guard"
    APP_VERSION = "0.12"
    LOGGING_LEVEL = LoggingLevels.DEBUG
    STORE_EXTENSION = ".sec"
    KEYRING_EXTENSION = ".skey"
    DEFAULT_SECRETS_PATH = os.path.join(str(Path.home()), ".secrets")






import copy
import json
import logging
import os
import re
from secrets_guard.crypt import aes_encrypt_file, aes_decrypt_file
from secrets_guard.utils import tabulate_enum, abort, highlight, enumerate_data

# A store is actually a file, encrypted with AES,
# which content is the following json model.
# Each element of "data" is called 'secret'.
# e.g.
# {
#   "model": [
#       {
#        "name": "Field1",
#        "hidden": true | false,
#        "mandatory: true | false
#       },
#       ...
#   ],
#   "data": [
#       {"Field1": "MyVal", "Field2": "AnotherVal"},
#       {"Field1": "MyVal", "Field2": "AnotherVal"},
#       {"Field1": "MyVal", "Field2": "AnotherVal"},
#       ...
#   ]
# }


# Each field of the model of the store is called StoreField
# e.g. Name or Account or Password)
class StoreField:

    class Json:
        NAME = "name"
        HIDDEN = "hidden"
        MANDATORY = "mandatory"

    def __init__(self, name, hidden=False, mandatory=False):
        self.name = name
        self.hidden = True if hidden else False
        self.mandatory = True if mandatory else False

    def __str__(self):
        return self.name + (" (hidden)" if self.hidden else "")

    def to_model(self):
        return {
            StoreField.Json.NAME: self.name,
            StoreField.Json.HIDDEN: self.hidden,
            StoreField.Json.MANDATORY: self.mandatory
        }

    @staticmethod
    def from_model(storefield_json):
        name = storefield_json.get(StoreField.Json.NAME, "")
        hidden = storefield_json.get(StoreField.Json.HIDDEN, None)
        mandatory = storefield_json.get(StoreField.Json.MANDATORY, None)
        sf = StoreField(name, hidden=hidden, mandatory=mandatory)
        return sf


class Store:

    class Json:
        MODEL = "model"
        DATA = "data"

    def __init__(self, path, name, key=None):
        self._path = path
        self._name = name

        self._key = key

        self._fullpath = os.path.join(path, name)
        self._fields = []
        self._secrets = []

    @property
    def name(self):
        return self._name

    @property
    def path(self):
        return self._path

    @property
    def key(self):
        return self._key

    def has_plain_key(self):
        """
        Returns whether the key in use is plaintext or already hashed.
        :return: whether the used key store is plaintext
        """
        return isinstance(self._key, str)

    @property
    def fullpath(self):
        return self._fullpath

    @property
    def fields(self):
        """
        Returns the store fields.
        :return: the store fields
        """
        return self._fields

    def fieldsnames(self):
        """
        Returns the store fields names.
        :return: the store fields names
        """

        names = []
        for f in self.fields:
            name = f.name
            # if f.hidden:
            #     name = "# " + name + " #"
            names.append(name)

        return names

    def add_fields(self, *fields):
        """
        Adds fields to the store (as StoreField).
        :param fields: the fields to add
        """

        for f in fields:
            self.fields.append(f)

        return True

    @property
    def secrets(self):
        """
        Returns the store data.
        :return: the store data
        """

        return self._secrets

    def secret(self, index):
        """
        Returns the secret at the given index.
        Note that  is is not the same as .secrets[index] since the secrets
        returned by secrets() are unordered.
        :param index: the secret index
        :return: the secret with the given index or None if it does not exist
        """
        ss = Store.sorted_secrets(self.secrets)
        if index < len(ss):
            return ss[index]
        return None

    def add_secrets(self, *secrets):
        """
        Adds secrets to the store.
        :param secrets: the secrets to add
        """

        for secret in secrets:
            safe_secret = {}
            self._apply_secret_change(safe_secret, secret)
            logging.info("Adding secret: %s", safe_secret)
            self.secrets.append(safe_secret)

        return True

    def remove_secrets(self, *secrets_ids):
        """
        Removes the secrets with the given id from the secrets.
        :param secrets_ids: the id of the secrets to remove
        :return whether the secret has been removed
        """

        logging.debug("Attempting secrets removal %s", secrets_ids)

        # Sort now since we must ensure that the user provided ID
        # matches the index of the secrets.
        self._secrets = Store.sorted_secrets(self.secrets)

        at_least_one_removed = False

        for secret_id in sorted(secrets_ids, reverse=True):

            logging.debug("Attempting to remove %s", secret_id)

            if secret_id >= len(self.secrets):
                logging.warning("Invalid secret id; out of bound")
                continue

            logging.info("Removing secret: %s", self.secrets[secret_id])

            del self.secrets[secret_id]
            at_least_one_removed = True

        return at_least_one_removed

    def clear_secrets(self):
        """
        Removes all  the store's secrets.
        """
        self._secrets = []

    def modify_secret(self, secret_id, secret_mod):
        """
        Modifies the secret with the given id using the given mod.
        :param secret_id: the secret it
        :param secret_mod: the new secret values
        :return whether the secret has been modified
        """

        if not secret_id < len(self.secrets):
            logging.error("Invalid secret id; out of bound")
            return False

        # Sort now since we must ensure that the user provided ID
        # matches the index of the secrets.
        self._secrets = Store.sorted_secrets(self.secrets)

        secret = self.secrets[secret_id]
        self._apply_secret_change(secret, secret_mod)

        return True

    def clone_content(self, store):
        """
        Clones the content (only the model and the data)
        of another store into this store.
        :param store: the store to copy
        """
        self._fields = store.fields
        self._secrets = store.secrets

    def destroy(self):
        """
        Destroys the store file associated with this store and free this store.
        :return: whether the store has been destroyed successfully.
        """
        logging.info("Destroying store at path '%s'", self._fullpath)

        if not os.path.exists(self._fullpath):
            logging.warning("Nothing to destroy, store does not exists")
            return False

        os.remove(self._fullpath)

        self._fields = []
        self._secrets = []

        return True

    def open(self, abort_on_fail=True):
        """
        Opens a store and parses the content into this store.
        :param abort_on_fail: whether abort if the store cannot be opened
        :return the store content
        """

        def do_store_open():
            logging.info("Opening store file at: %s", self._fullpath)
            logging.debug("Using key '%s' ; plainkey? %s", self._key, self.has_plain_key())

            if not os.path.exists(self._fullpath):
                logging.error("Path does not exist")
                return None

            store_content = aes_decrypt_file(self._fullpath, self._key,
                                             is_plain_key=self.has_plain_key())

            if not store_content:
                return None

            logging.debug("Store opened; content is: \n%s", store_content)
            try:
                store_json = json.loads(store_content)
                logging.debug("Store parsed content is: %s", store_json)
                if not Store.is_valid_store_json(store_json):
                    logging.error("Invalid store content")
                    return None
            except ValueError:
                logging.error("Invalid store content")
                return None

            return store_json

        jstore = do_store_open()

        if abort_on_fail and not jstore:
            abort("Error: unable to open store '%s'" % self._name)

        # Parse the content
        self._parse_model(jstore)

        return jstore is not None

    def save(self):
        """
        Writes the current store content to the store file.
        :return: whether the store has been written successfully
        """

        logging.info("Writing store file at: %s", self._fullpath)

        if not os.path.exists(self._path):
            logging.debug("Creating path %s since it does not exists", self._path)
            try:
                os.makedirs(self._path)
            except OSError:
                logging.warning("Exception occurred, cannot create directory")
                return False

        logging.debug("Actually flushing store %s, \nModel: %s \nSecrets: %s",
                      self._fullpath, self.fieldsnames(), self.secrets)

        write_ok = aes_encrypt_file(self._fullpath, self._key,
                                    json.dumps(self.to_model()),
                                    is_plain_key=self.has_plain_key())

        return write_ok and os.path.exists(self._fullpath)

    def show(self, table=True):
        """
        Prints the data of the store as tabulated data.
        :param table: whether show the data within a table
        :return: whether the store has been printed successfully
        """
        if table:
            print(tabulate_enum(self.fieldsnames(), Store.sorted_secrets(self.secrets)))
        else:
            print(Store.list_secrets(self.fieldsnames(), Store.sorted_secrets(self.secrets)))

        return True

    def grep(self, grep_pattern, colors=True, table=True):
        """
        Performs a regular expression between each field of each secret and
        prints the matches a tabular data.
        :param grep_pattern: the search pattern as a valid regular expression
        :param colors: whether highlight the matches
        :param table: whether show the data within a table
        :return: whether the secret has been grep-ed successfully
        """

        matches = []
        for i, secret in enumerate(Store.sorted_secrets(self.secrets)):
            secretmatch = None
            for field in secret:
                # logging.debug("Comparing %s against %s", field, grep_pattern)

                re_matches = re.finditer(grep_pattern, secret[field], re.IGNORECASE)

                for re_match in re_matches:
                    startpos, endpos = re_match.span()
                    logging.debug("Found re match in: %s ", secret[field])

                    if colors:
                        # Do a copy, leave 'secret' as it is
                        if not secretmatch:
                            secretmatch = copy.copy(secret)

                        # Highlight
                        secretmatch[field] = highlight(secret[field], startpos, endpos)
                    else:
                        secretmatch = secret

            if secretmatch:
                secretmatch["ID"] = i
                matches.append(secretmatch)

        logging.debug("There are %d matches", len(matches))

        if table:
            print(tabulate_enum(self.fieldsnames(), matches))
        else:
            print(Store.list_secrets(self.fieldsnames(), matches))

        return True

    def _apply_secret_change(self, secret, secret_mod):
        """
        For each known field of store_fields push the value from secret_mod
        to secret.
        :param secret: the secret
        :param secret_mod: the secret modification (may contain only some fields)
        """
        store_fields = self.fieldsnames()

        logging.debug("Applying secret mod.\nCurrent: %s\nMod: %s", secret, secret_mod)

        for store_field in store_fields:
            for mod_field in secret_mod:
                if store_field.lower() == mod_field.lower():
                    secret[store_field] = secret_mod[mod_field]

        logging.debug("Secret after mod: %s", secret)

    def _parse_model(self, store_model):
        """
        Parse the json content and fill the fields and secrets of this store
        accordingly.
        :param store_model: the dictionary of the store as json
        """

        logging.debug("Parsing store model %s", store_model)

        if not Store.is_valid_store_json(store_model):
            logging.warning("Invalid store json")
            return

        self._fields = [StoreField.from_model(field) for field in store_model[Store.Json.MODEL]]
        self._secrets = store_model[Store.Json.DATA]

    def to_model(self):
        """
        Returns a json dictionary representing this store.
        :return: this store as a json dictionary
        """
        return {
            Store.Json.MODEL: [f.to_model() for f in self.fields],
            Store.Json.DATA: self.secrets
        }

    def __str__(self):
        s = "Store path: " + self._fullpath + "\n"

        s += "Fields: "

        for field in self.fields:
            s += field

        s += "\n"

        s += "Secrets: "

        for secret in self.secrets:
            s += secret

        return s

    @staticmethod
    def sorted_secrets(secrets):
        """
        Returns the secrets sorted by fields (in the order of the fields).
        :return: the store data sorted
        """

        def lowered(tup):
            return [str(f).lower() for f in tup]

        return sorted(secrets, key=lambda s: [lowered(t) for t in list(s.items())])

    @staticmethod
    def is_valid_store_json(j):
        """
        Returns whether the given json is a valid json for a store.
        :param j: the json to check
        :return: whether the json could be the json of a store
        """
        return j and Store.Json.MODEL in j and Store.Json.DATA in j

    @staticmethod
    def list_secrets(headers, secrets):
        """
        Returns a the content of the secrets, field per field.
        :param headers: the store headers
        :param secrets: the secrets
        :return: a string representing the secrets and their content
        """
        s = ""
        enum_headers, enum_data = enumerate_data(headers, secrets)

        for di, d in enumerate(enum_data):
            is_last_data = di >= len(enum_data) - 1

            for hi, h in enumerate(enum_headers):
                is_last_header = hi >= len(enum_headers) - 1

                if h not in d:
                    continue
                s += h + ": " + str(d[h])
                if not is_last_data or not is_last_header:
                    s += "\n"

            if not is_last_data:
                s += "-" * 20 + "\n"

        return s



import logging
import os
import tempfile

from secrets_guard.conf import Conf
from secrets_guard.crypt import aes_key


# Basic implementation of a keyring.
#
# The goal is to not retype the password over and over again for multiple
# actions on the same store.
# This is achieved by saving an hash of the key into a file with the same name
# of the store in the system temporary folder (e.g. /tmp/password.skey).
# This guarantees that the keyring is freed-up on the next system boot.
# The content of the file is the actually the hashed key.

def keyring_put_key(store_name, store_key, is_plain_key=True):
    """
    Puts the given key (plain or already hashed) in the system temporary folder
    for further uses.
    :param store_name: the store name
    :param store_key: the store key
    :param is_plain_key: whether the key is plaintext or already hashed
    """
    keyring_path = os.path.join(tempfile.gettempdir(), store_name + Conf.KEYRING_EXTENSION)
    with open(keyring_path, "wb") as keyring:
        logging.info("Adding key to the keyring for store '%s'" % store_name)
        keyring.write(aes_key(store_key) if is_plain_key else store_key)


def keyring_has_key(store_name):
    """
    Returns whether the key for the given store exists in the keyring.
    :param store_name: the store name
    :return: whether the key exists
    """
    keyring_path = os.path.join(tempfile.gettempdir(), store_name + Conf.KEYRING_EXTENSION)
    return os.path.exists(keyring_path)


def keyring_get_key(store_name):
    """
    Returns the cached hash of the key for the store with the given name,
    or None if does not exist.
    :param store_name: the store name
    :return: the (hashed) store key
    """
    keyring_path = os.path.join(tempfile.gettempdir(), store_name + Conf.KEYRING_EXTENSION)
    if not keyring_has_key(store_name):
        return None

    with open(keyring_path, "rb") as keyring:
        logging.debug("Getting key from the keyring for store '%s'" % store_name)
        return keyring.read()


def keyring_del_key(store_name):
    """
    Deletes the key for the given store name from the keyring.
    :param store_name: the store name
    """
    keyring_path = os.path.join(tempfile.gettempdir(), store_name + Conf.KEYRING_EXTENSION)
    if os.path.exists(keyring_path):
        logging.info("Removing key from the keyring for store '%s'" % store_name)
        os.remove(keyring_path)




import os
from setuptools import setup, find_packages

from secrets_guard.conf import Conf


def read(file_name):
    with open(os.path.join(os.path.dirname(__file__), file_name)) as f:
        return f.read()


setup(
    name=Conf.APP_NAME,
    version=Conf.APP_VERSION,

    # Requires python3.5
    python_requires=">=3",

    # Automatically import packages
    packages=find_packages(),

    include_package_data=True,

    # Scripts to install to the user executable path.
    # Note that this might be something like /home/user/.local/bin
    # which in Debian distributions is not included in $PATH.
    # If you want to use just "compress" or "uncompress", you should add that
    # path to your $PATH.
    entry_points={
      'console_scripts': [
          'secrets=secrets_guard.main:main'
      ]
    },

    # Tests
    test_suite="tests",

    # Dependencies
    install_requires=['pycryptodome', 'gitpython'],

    # Metadata
    author="Stefano Dottore",
    author_email="docheinstein@gmail.com",
    description="Encrypts and decrypts private information",
    long_description_content_type="text/x-rst",
    long_description=read('README.rst'),
    license="MIT",
    keywords="pass password private key encrypt decrypt crypt",
    url="https://github.com/Docheinstein/secrets-guard",
)



import random
import tempfile
import unittest

from Crypto import Random
from Crypto.Cipher import AES

from secrets_guard.crypt import aes_encrypt_file, aes_decrypt_file, aes_encrypt, aes_decrypt
from secrets_guard.utils import random_string


class AesTests(unittest.TestCase):

    ITERS = 256

    def test_encrypt_decrypt(self):
        for i in range(AesTests.ITERS):

            original_text = random_string(random.randint(1, 8192))
            key = random_string(random.randint(1, 50))
            iv = Random.new().read(AES.block_size)

            cipher_text = aes_encrypt(original_text, iv, key)

            decrypted_text = aes_decrypt(cipher_text, key)

            self.assertEqual(original_text, decrypted_text)

    def test_encrypt_decrypt_file(self):
        fd, fname = tempfile.mkstemp()

        for i in range(AesTests.ITERS):
            original_text = random_string(random.randint(1, 8192))
            key = random_string(random.randint(1, 50))

            aes_encrypt_file(fname, key, original_text)
            decrypted_text = aes_decrypt_file(fname, key)

            self.assertEqual(original_text, decrypted_text)


if __name__ == "__main__":
    unittest.main()





